//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGLWindow.cpp
// Description : Classe contenant les informations d'une fenetre
//===========================================================================


#include <GL/glut.h>
#include <iostream>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <qmessagebox.h>
#include "./GLWindow.h"

using namespace std ;


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CGLWindow::CGLWindow( void )
{
   // shaders
   m_pShaderManager   = NULL ;
   m_pAdvancedEdgesVS = NULL ;
   m_pAdvancedEdgesPS = NULL ;

   // effet
   m_pEffect = NULL ;

   // texture
   m_pStyleGLTex = NULL ;
}

//-------------------------------------------------------------------------

CGLWindow::~CGLWindow( void )
{
   // shaders (le shader manager detruit tous les shaders qu'il manipule)
   if( NULL != m_pShaderManager ) { delete m_pShaderManager ; }

   // effet
   if( NULL != m_pEffect ) { delete m_pEffect ; }

   // texture
   if( NULL != m_pStyleGLTex ) { delete m_pStyleGLTex ; }
}






//-------------------------------------------------------------------------
// ACTIONS ----------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::draw( void )
{
   // affacement des buffers
   glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT ) ;

   // affichage de la scene pour chaque passe
   for( CCGPass * pass = m_pEffect->getFirstPass() ;
        pass != NULL ;
	pass = m_pEffect->getNextPass()
      )
   {
      pass->enable() ;
      m_oDrawer.draw( m_oMeshScene.getMesh() ) ;
      pass->disable() ;
   }
}

//-------------------------------------------------------------------------

void CGLWindow::init( void )
{
   // chargement de la premiere scene
   m_oMeshScene.load( "../models/badguyfin.x3d" ) ;
   setSceneBoundingBox( m_oMeshScene.getBBoxMin().f_data(),
                        m_oMeshScene.getBBoxMax().f_data() ) ;
   showEntireScene() ;

   // creation du manageur de shaders
   m_pShaderManager = new CCGShaderManager ;

   // creation des shaders
   {
      m_pAdvancedEdgesVS = new CCGAdvancedEdgesVS(
            m_pShaderManager->getGlobalContext(),  // contexte Cg
            "AdvancedEdgesVP.cg"                ,  // fichier du vertex program
	    "AdvancedEdgesVP"                      // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingVertexShader(
            m_pAdvancedEdgesVS  ,                  // Vertex shader a manager
	    "AdvancedEdgesVP.cg"                   // nom du VS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }

      // pixel shader
      m_pAdvancedEdgesPS = new CCGAdvancedEdgesPS(
            m_pShaderManager->getGlobalContext(),  // contexte Cg
	    "AdvancedEdgesFP.cg"                ,  // fichier du fragment program
	    "AdvancedEdgesFP"                      // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingPixelShader(
            m_pAdvancedEdgesPS  ,                  // pixel shader a manager
	    "AdvancedEdgesFP.cg"                   // nom du PS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }
   }

   // preparation des effets
   {
      // creation des parametres :
      // on donne pour chaque parametre le nom qui lui correspond dans le
      // programme Cg. Ces nom etant arbitraires (a moins d'imposer une norme
      // de nommage) il est impossible de les deviner.
      m_pAdvancedEdgesVS->setModelViewProjMatrixParamName( "modelViewProjMatrix" ) ;
      m_pAdvancedEdgesVS->setEyePositionParamName(         "eyePosition"         ) ;

      m_pAdvancedEdgesPS->setEdgeTexParamName(    "edgeTex"    ) ;
      m_pAdvancedEdgesPS->setConstColorParamName( "constColor" ) ;

      // mise en place de la texture
      glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
      m_pStyleGLTex = new CGLTexture( "../textures/Edges2D_points.png" ) ;
      m_pStyleGLTex->createGLTexture( GL_REPEAT, GL_REPEAT ) ;

      // initialisation des parametres constants
      m_oConstColor.setRgb( 255, 0, 255 ) ;
      GLfloat constColor[] = { 1.0f, 0.0f, 1.0f } ;
      m_pAdvancedEdgesPS->setCU_ConstColor( constColor ) ;
      m_pAdvancedEdgesPS->setCU_EdgeTex( m_pStyleGLTex->getGLTexName() ) ;

      // initialisation des parametres d'entree
      // pour pouvoir recuperer la position de la camera a chaque image,
      // il nous faut donner la camera elle meme.
      m_pAdvancedEdgesVS->setU_EyePosition( camera() ) ;
   }
   
   // creation de l'effet associe
   m_pEffect = new CCGEffect ;
   m_pEffect->pushPass(
         m_pAdvancedEdgesVS,    // Vertex Shader de la passe 1
	 m_pAdvancedEdgesPS,    // Pixel Shader de la passe 1
	 true              ,    // activation du test de profondeur
	 true              ,    // activation de l'ecriture dans le Z buffer
	 false             ,    // pas de test d'alpha
	 false                  // pas de blending
      ) ;

   // initialisation d'OpenGL et du mode d'affichage
   glEnable( GL_TEXTURE_2D ) ;
   m_oDrawer.setRenderingMode( X3DTK::MESH::REAL ) ;
   setBackgroundColor( 255, 255, 255 ) ;
}

//-------------------------------------------------------------------------

void CGLWindow::keyPressEvent( QKeyEvent * e )
{
   switch( e->key() )
   {
      // chargement d'un nouveau modele X3D
      case Qt::Key_L :
      {
         loadSceneFile() ;
      }
      break ;

      // chargement d'une nouvelle texture de style
      case Qt::Key_S :
      {
         loadStyleFile() ;
      }
      break ;

      // changement de la couleur constante
      case Qt::Key_C :
      {
         changeConstColor() ;
      }
      break ;

      default :
      {
         QGLViewer::keyPressEvent( e ) ;
      }
   }
}

//-------------------------------------------------------------------------

void CGLWindow::help( void ) const
{

   QMessageBox mesg( "help"                                ,
                     helpString()                          ,
                     QMessageBox::NoIcon                   ,
		     QMessageBox::Ok | QMessageBox::Default,
		     QMessageBox::NoButton                 ,
		     QMessageBox::NoButton                 ,
		     NULL                                  ,
		     "Help"                                ,
		     false                                 ,
		       Qt::WStyle_DialogBorder
		     | Qt::WType_Dialog
		     | Qt::WDestructiveClose                ) ;
   mesg.show() ;
}






//-------------------------------------------------------------------------
// PARAMETRAGE ------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::loadSceneFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                      ,
         "X3D files (*.x3d *.X3D);;All files (*)",
         this
      ) ;

   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement du modele et modification de la scene en consequence
   m_oMeshScene.load( filename ) ;
   setSceneBoundingBox( m_oMeshScene.getBBoxMin().f_data(),
                        m_oMeshScene.getBBoxMax().f_data() ) ;
   showEntireScene() ;
}

//-------------------------------------------------------------------------

void CGLWindow::loadStyleFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                        ,
         "Image files (*.png *.jpg);;All files (*)",
         this
      ) ;
   
   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement de la texture et modification des shaders en consequence
   if( m_pStyleGLTex != NULL ) { delete m_pStyleGLTex ; }
   m_pStyleGLTex = new CGLTexture( filename ) ;
   m_pStyleGLTex->createGLTexture() ;
   m_pAdvancedEdgesPS->setCU_EdgeTex( m_pStyleGLTex->getGLTexName() ) ;
}

//-------------------------------------------------------------------------

void CGLWindow::changeConstColor( void )
{
   // recuperation de la valeur
   m_oConstColor = QColorDialog::getColor( m_oConstColor, this ) ;

   // modification des shaders en consequence
   float constColor[3] = { (float)m_oConstColor.red()   / 255.0f,
                           (float)m_oConstColor.green() / 255.0f,
			   (float)m_oConstColor.blue()  / 255.0f } ;
   m_pAdvancedEdgesPS->setCU_ConstColor( constColor ) ;
}






//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

QString CGLWindow::helpString( void ) const
{
   QString message( "" ) ;
  
   message += "<b>L</b> loads a new scene file<br>" ;
   message += "<b>S</b> loads a new style file<br>" ;
   message += "<b>C</b> change constant color<br>"  ;

   message += QGLViewer::helpString() ;
  
   return message ;
}

//-------------------------------------------------------------------------
