//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGLWindow.cpp
// Description : Classe contenant les informations d'une fenetre
//===========================================================================


#include <GL/glut.h>
#include <iostream>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include "./GLWindow.h"

using namespace std ;


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CGLWindow::CGLWindow( void )
{
   // shaders
   m_pShaderManager = NULL ;
   m_pBasicEdgesVS  = NULL ;
   m_pBasicEdgesPS  = NULL ;

   // effet
   m_pEffect = NULL ;

   // texture
   m_pEdgeGLTex = NULL ;
}

//-------------------------------------------------------------------------

CGLWindow::~CGLWindow( void )
{
   // shaders (le shader manager detruit tous les shaders qu'il manipule)
   if( NULL != m_pShaderManager ) { delete m_pShaderManager ; }

   // effet
   if( NULL != m_pEffect ) { delete m_pEffect ; }

   // texture
   if( NULL != m_pEdgeGLTex ) { delete m_pEdgeGLTex ; }
}






//-------------------------------------------------------------------------
// ACTIONS ----------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::draw( void )
{
   // affacement des buffers
   glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT ) ;

   // affichage de la scene pour chaque passe
   for( CCGPass * pass = m_pEffect->getFirstPass() ;
        pass != NULL ;
	pass = m_pEffect->getNextPass()
      )
   {
      pass->enable() ;
      m_oDrawer.draw( m_oMeshScene.getMesh() ) ;
      pass->disable() ;
   }
}

//-------------------------------------------------------------------------

void CGLWindow::init( void )
{
   // chargement de la premiere scene
   m_oMeshScene.load( "../models/porsche.x3d" ) ;
   setSceneBoundingBox( m_oMeshScene.getBBoxMin().f_data(),
                        m_oMeshScene.getBBoxMax().f_data() ) ;
   showEntireScene() ;

   // creation du manageur de shaders
   m_pShaderManager = new CCGShaderManager ;

   // creation des shaders
   {
      m_pBasicEdgesVS = new CCGBasicEdgesVS(
            m_pShaderManager->getGlobalContext(),  // contexte Cg
            "BasicEdgesVP.cg"                   ,  // fichier du vertex program
	    "BasicEdgesVP"                         // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingVertexShader(
            m_pBasicEdgesVS  ,                     // Vertex shader a manager
	    "BasicEdgesVP.cg"                      // nom du VS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }

      m_pBasicEdgesPS = new CCGBasicEdgesPS(
            m_pShaderManager->getGlobalContext(),  // contexte Cg
	    "BasicEdgesFP.cg"                   ,  // fichier du fragment program
	    "BasicEdgesFP"                         // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingPixelShader(
            m_pBasicEdgesPS  ,                     // pixel shader a manager
	    "BasicEdgesFP.cg"                      // nom du PS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }
   }

   // preparation des effets
   {
      // creation des parametres :
      // on donne pour chaque parametre le nom qui lui correspond dans le
      // programme Cg. Ces nom etant arbitraires (a moins d'imposer une norme
      // de nommage) il est impossible de les deviner.
      m_pBasicEdgesVS->setModelViewProjMatrixParamName( "modelViewProjMatrix" ) ;
      m_pBasicEdgesVS->setEyePositionParamName(         "eyePosition"         ) ;

      m_pBasicEdgesPS->setEdgeTexParamName( "edgeTex" ) ;

      // mise en place de la texture
      glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
      m_pEdgeGLTex = new CGLTexture( "../textures/edges.png" ) ;
      m_pEdgeGLTex->createGLTexture( GL_REPEAT, GL_REPEAT ) ;

      // initialisation des parametres constants
      m_pBasicEdgesPS->setCU_EdgeTex( m_pEdgeGLTex->getGLTexName() ) ;

      // initialisation des parametres d'entree
      // pour pouvoir recuperer la position de la camera a chaque image,
      // il nous faut donner la camera elle meme.
      m_pBasicEdgesVS->setU_EyePosition( camera() ) ;
   }

   // creation de l'effet associe
   m_pEffect = new CCGEffect ;
   m_pEffect->pushPass(
         m_pBasicEdgesVS,    // Vertex Shader de la passe 1
	 m_pBasicEdgesPS,    // Pixel Shader de la passe 1
	 true           ,    // activation du test de profondeur
	 true           ,    // activation de l'ecriture dans le Z buffer
	 false          ,    // pas de test d'alpha
	 false               // pas de blending
      ) ;

   // initialisation d'OpenGL et du mode d'affichage
   glEnable( GL_TEXTURE_1D ) ;
   m_oDrawer.setRenderingMode( X3DTK::MESH::REAL ) ;
   setBackgroundColor( 255, 255, 255 ) ;
}

//-------------------------------------------------------------------------

void CGLWindow::keyPressEvent( QKeyEvent * e )
{
   switch( e->key() )
   {
      // chargement d'un nouveau modele X3D
      case Qt::Key_L :
      {
         loadSceneFile() ;
      }
      break ;

      // chargement d'une nouvelle texture de bord
      case Qt::Key_E :
      {
         loadEdgeFile() ;
      }
      break ;

      default :
      {
         QGLViewer::keyPressEvent( e ) ;
      }
   }
}

//-------------------------------------------------------------------------

void CGLWindow::help( void ) const
{

   QMessageBox mesg( "help"                                ,
                     helpString()                          ,
                     QMessageBox::NoIcon                   ,
		     QMessageBox::Ok | QMessageBox::Default,
		     QMessageBox::NoButton                 ,
		     QMessageBox::NoButton                 ,
		     NULL                                  ,
		     "Help"                                ,
		     false                                 ,
		       Qt::WStyle_DialogBorder
		     | Qt::WType_Dialog
		     | Qt::WDestructiveClose                ) ;
   mesg.show() ;
}

//-------------------------------------------------------------------------
// PARAMETRAGE ------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::loadSceneFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                      ,
         "X3D files (*.x3d *.X3D);;All files (*)",
         this
      ) ;

   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement du modele et modification de la scene en consequence
   m_oMeshScene.load( filename ) ;
   setSceneBoundingBox( m_oMeshScene.getBBoxMin().f_data(),
                        m_oMeshScene.getBBoxMax().f_data() ) ;
   showEntireScene() ;
}

//-------------------------------------------------------------------------

void CGLWindow::loadEdgeFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                        ,
         "Image files (*.png *.jpg);;All files (*)",
         this
      ) ;
   
   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement de la texture et modification des shaders en consequence
   if( m_pEdgeGLTex != NULL ) { delete m_pEdgeGLTex ; }
   m_pEdgeGLTex = new CGLTexture( filename ) ;
   m_pEdgeGLTex->createGLTexture() ;
   m_pBasicEdgesPS->setCU_EdgeTex( m_pEdgeGLTex->getGLTexName() ) ;
}






//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

QString CGLWindow::helpString( void ) const
{
   QString message( "" ) ;
  
   message += "<b>L</b> loads a new scene file<br>" ;
   message += "<b>E</b> loads a new style file<br>" ;

   message += QGLViewer::helpString() ;
  
   return message ;
}

//-------------------------------------------------------------------------
