//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGLWindow.cpp
// Description : Classe contenant les informations d'une fenetre
//===========================================================================


#include <GL/glut.h>
#include <iostream>
#include <qfiledialog.h> 
#include <qmessagebox.h>
#include <qinputdialog.h>
#include "./GLWindow.h"

using namespace std ;


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CGLWindow::CGLWindow( void )
{
   // shaders
   m_pShaderManager = NULL ;
   m_pSketchVS      = NULL ;
   m_pSketchPS      = NULL ;

   // effet
   m_pEffect = NULL ;

   // textures
   m_pBrushGLTex      = NULL ;
   m_pBackgroundGLTex = NULL ;
}

//-------------------------------------------------------------------------

CGLWindow::~CGLWindow( void )
{
   // shaders (le shader manager detruit tous les shaders qu'il manipule)
   if( NULL != m_pShaderManager ) { delete m_pShaderManager ; }

   // effet
   if( NULL != m_pEffect ) { delete m_pEffect ; }

   // textures
   if( NULL != m_pBrushGLTex      ) { delete m_pBrushGLTex      ; }
   if( NULL != m_pBackgroundGLTex ) { delete m_pBackgroundGLTex ; }
}






//-------------------------------------------------------------------------
// ACTIONS ----------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::draw( void )
{
   // affacement des buffers
   glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT ) ;

   // affichage de la sky box
   glMatrixMode( GL_MODELVIEW ) ;
   glPushMatrix() ;
      
      // on ne sauve pas les attributs puisqu'il n'influence pas l'effet
      glDisable( GL_DEPTH_TEST ) ;
      glDisable( GL_LIGHTING   ) ;
      glDisable( GL_DEPTH_TEST ) ;
      glDisable( GL_BLEND      ) ;
      glEnable(  GL_TEXTURE_2D ) ;
      
      glDepthMask( 0 ) ;
      glTranslatef( camera()->position()[0],
                    camera()->position()[1],
                    camera()->position()[2] ) ;
      glBindTexture( GL_TEXTURE_2D, m_pBackgroundGLTex->getGLTexName() ) ;
      glColor3f( 1.0f, 1.0f, 1.0f ) ;
      glBegin( GL_QUADS ) ;
         glCallList( m_uiSkyBoxList ) ;
      glEnd() ;
   glPopMatrix() ;
   

   // on affiche la scene pour chaque passe de l'effet
   // les passes s'occupent de la fusion entre chaque affichage
   glBindTexture( GL_TEXTURE_2D, m_pBrushGLTex->getGLTexName() ) ;
   
   for( CCGPass * pass = m_pEffect->getFirstPass() ;
        pass != NULL ;
	pass = m_pEffect->getNextPass()
      )
   {
      pass->enable() ;
      m_oDrawer.draw( m_oMeshScene.getMesh() ) ;
      pass->disable() ;
   }
}

//-------------------------------------------------------------------------

void CGLWindow::init( void )
{
   // chargement de la premiere scene
   m_oMeshScene.load( "../models/porsche.x3d" ) ;
   setSceneBoundingBox( m_oMeshScene.getBBoxMin().f_data(),
                        m_oMeshScene.getBBoxMax().f_data() ) ;
   showEntireScene() ;

   // creation du manageur de shaders
   m_pShaderManager = new CCGShaderManager ;

   // creation des shaders
   {
      m_pSketchVS = new CCGSketchVS(
            m_pShaderManager->getGlobalContext(), // contexte Cg
            "SketchVP.cg"                       , // fichier du vertex program
	    "SketchVP"                            // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingVertexShader(
            m_pSketchVS  ,                        // Vertex shader a manager
	    "SketchVP.cg"                         // nom du VS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }

      m_pSketchPS = new CCGSketchPS(
            m_pShaderManager->getGlobalContext(), // contexte Cg
	    "SketchFP.cg"                       , // fichier du fragment program
	    "SketchFP"                            // point d'entree du programme
	 ) ;
      m_pShaderManager->manageExistingPixelShader(
            m_pSketchPS  ,                        // pixel shader a manager
	    "SketchFP.cg"                         // nom du PS pour le referencer
	 ) ;

      // on affiche les erreurs de compilation s'il y en a
      if( NULL != m_pShaderManager->getLastListing() )
      {
	 cout << m_pShaderManager->getLastListing() << endl ;
      }
   }

   // preparation des effets
   {
      // creation des parametres
      // on donne pour chaque parametre le nom qui lui correspond dans le
      // programme Cg. Ces nom etant arbitraires (a moins d'imposer une norme
      // de nommage) il est impossible de les deviner.
      m_pSketchVS->setModelViewProjMatrixParamName( "modelViewProjMatrix" ) ;
      m_pSketchVS->setEyePositionParamName(         "eyePosition"         ) ;
      m_pSketchVS->setDisplacmentParamName(         "depl"                ) ;
      m_pSketchVS->setFactorParamName(              "fact"                ) ;

      m_pSketchPS->setBrushTexParamName(   "brushTex"   ) ;
      m_pSketchPS->setConstColorParamName( "constColor" ) ;

      // mise en place des textures
      glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ) ;
      m_pBrushGLTex = new CGLTexture( "../textures/Drawed_slash.png" ) ;
      m_pBrushGLTex->createGLTexture() ;

      // initialisation des parametres constants
      m_fDisplacment = 0.1f  ;
      m_fFactor      = 0.05f ;
      m_pSketchVS->setCU_Displacment( m_fDisplacment ) ;
      m_pSketchVS->setCU_Factor(      m_fFactor      ) ;

      m_oConstColor.setRgb( 255, 255, 255 ) ;
      float firstColor[3] = { 1.0f, 1.0f, 1.0f } ;
      m_pSketchPS->setCU_ConstColor( firstColor ) ;
      m_pSketchPS->setCU_BrushTex( m_pBrushGLTex->getGLTexName() ) ;

      // initialisation des parametres d'entree
      m_pSketchVS->setU_EyePosition( camera() ) ;
   }

   // creation de l'effet associe
   m_pEffect = new CCGEffect ;
   m_pEffect->pushPass(
         m_pSketchVS,    // Vertex Shader de la passe 1
	 m_pSketchPS,    // Pixel Shader de la passe 1
	 true       ,    // activation du test de profondeur
	 true       ,    // activation de l'ecriture dans le Z buffer
	 false      ,    // pas de test d'alpha
	 true            // activation du blending pour l'effet dessin
      ) ;

   // creation de l'image de fond
   m_pBackgroundGLTex = new CGLTexture( "../textures/background.png" ) ;
   m_pBackgroundGLTex->createGLTexture(
         GL_REPEAT,
	 GL_REPEAT,
	 GL_LINEAR,
	 GL_LINEAR
      ) ;
   
   // creation de la liste d'affichage pour la sky box
   m_uiSkyBoxList = glGenLists( 1 ) ;
   glNewList( m_uiSkyBoxList, GL_COMPILE ) ;
      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f( -0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f(  0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f(  0.5f,  0.5f,  0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f( -0.5f,  0.5f,  0.5f ) ;

      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f(  0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f( -0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f( -0.5f, -0.5f, -0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f(  0.5f, -0.5f, -0.5f ) ;

      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f(  0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f(  0.5f, -0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f(  0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f(  0.5f,  0.5f,  0.5f ) ;

      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f( -0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f( -0.5f,  0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f( -0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f( -0.5f, -0.5f, -0.5f ) ;

      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f( -0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f(  0.5f,  0.5f, -0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f(  0.5f,  0.5f,  0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f( -0.5f,  0.5f,  0.5f ) ;

      glTexCoord2f( 0.0f, 0.0f ) ; glVertex3f(  0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 0.0f ) ; glVertex3f( -0.5f, -0.5f,  0.5f ) ;
      glTexCoord2f( 2.0f, 2.0f ) ; glVertex3f( -0.5f, -0.5f, -0.5f ) ;
      glTexCoord2f( 0.0f, 2.0f ) ; glVertex3f(  0.5f, -0.5f, -0.5f ) ;
   glEndList() ;	

   // initialisation d'OpenGL et du mode d'affichage
   glEnable( GL_TEXTURE_1D ) ;
   glEnable( GL_TEXTURE_2D ) ;
   m_oDrawer.setRenderingMode( X3DTK::MESH::REAL ) ;
   setBackgroundColor( 255, 255, 255 ) ;

   // on met une grande BB pour ne pas voir disparaitre la sky box ...
   // il y a surement un autre moyen de faire cela !!
   float min[3] = { -200.0f, -200.0f, -200.0f } ;
   float max[3] = {  200.0f,  200.0f,  200.0f } ;
   setSceneBoundingBox( min, max ) ;
}

//-------------------------------------------------------------------------

void CGLWindow::keyPressEvent( QKeyEvent * e )
{
   switch( e->key() )
   {
      // chargement d'un nouveau modele X3D
      case Qt::Key_L :
      {
         loadSceneFile() ;
      }
      break ;

      // chargement d'une nouvelle texture de dessin
      case Qt::Key_B :
      {
         loadBrushFile() ;
      }
      break ;

      // changement de la valeur de deplacement des vertex de bords
      case Qt::Key_D :
      {
         changeDisplacment() ;
      }
      break ;

      // changement de la valeur du decalage initial dans la texture de dessin
      case Qt::Key_F :
      {
         changeFactor() ;
      }
      break ;

      default :
      {
         QGLViewer::keyPressEvent( e ) ;
      }
   }
}

//-------------------------------------------------------------------------

void CGLWindow::help( void ) const
{

   QMessageBox mesg( "help"                                ,
                     helpString()                          ,
                     QMessageBox::NoIcon                   ,
		     QMessageBox::Ok | QMessageBox::Default,
		     QMessageBox::NoButton                 ,
		     QMessageBox::NoButton                 ,
		     NULL                                  ,
		     "Help"                                ,
		     false                                 ,
		       Qt::WStyle_DialogBorder
		     | Qt::WType_Dialog
		     | Qt::WDestructiveClose                ) ;
   mesg.show() ;
}






//-------------------------------------------------------------------------
// PARAMETRAGE ------------------------------------------------------------
//-------------------------------------------------------------------------

void CGLWindow::loadSceneFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                      ,
         "X3D files (*.x3d *.X3D);;All files (*)",
         this
      ) ;

   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement du modele et modification de la scene en consequence
   m_oMeshScene.load( filename ) ;
   showEntireScene() ;
}

//-------------------------------------------------------------------------

void CGLWindow::loadBrushFile( void )
{
   // recuperation du nom du fichier a charger
   QString filename = QFileDialog::getOpenFileName(
         ""                                        ,
         "Image files (*.png *.jpg);;All files (*)",
         this
      ) ;
   
   // dans le cas ou on appuie sur ANNULER
   if( filename.isEmpty() ) { return ; }

   // chargement de la texture et modification des shaders en consequence
   if( m_pBrushGLTex != NULL ) { delete m_pBrushGLTex ; }
   m_pBrushGLTex = new CGLTexture( filename ) ;
   m_pBrushGLTex->createGLTexture() ;
   m_pSketchPS->setCU_BrushTex( m_pBrushGLTex->getGLTexName() ) ;
}

//-------------------------------------------------------------------------

void CGLWindow::changeDisplacment( void )
{
   // recuperation de la valeur
   bool ok ;
   m_fDisplacment = (float)QInputDialog::getDouble(
         ""                    ,
         "Displacment value"   ,
         (double)m_fDisplacment,
         0.0                   ,
         8.0                   ,
         4                     ,
         &ok                   ,
         this
      ) ;

   // modification des shaders en consequence
   if( ok ) { m_pSketchVS->setCU_Displacment( m_fDisplacment ) ; }
}

//-------------------------------------------------------------------------

void CGLWindow::changeFactor( void )
{
   // recuperation de la valeur
   bool ok ;
   m_fFactor = (float)QInputDialog::getDouble(
         ""               ,
         "Factor value"   ,
         (double)m_fFactor,
         0.0              ,
         1.0              ,
         4                ,
         &ok              ,
         this
      ) ;

   // modification des shaders en consequence
   if( ok ) { m_pSketchVS->setCU_Factor( m_fFactor ) ; }
}






//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

QString CGLWindow::helpString( void ) const
{
   QString message( "" ) ;
  
   message += "<b>L</b> loads a new scene file<br>"   ;
   message += "<b>B</b> loads a new brush file<br>"   ;
   message += "<b>D</b> change displacment value<br>" ;
   message += "<b>F</b> change factor value<br>"      ;

   message += QGLViewer::helpString() ;
  
   return message ;
}

//-------------------------------------------------------------------------
